#
#  Copyright 2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

set MxOptimize::show_detune_button 1
set MxOptimize::have_servo_control 1
set MxOptimize::scans_autoscale_scaler 1

proc create_optimize_gui { record_list } {
	
	frame .title
	pack .title -before .message_log
	
	label .title.text -text "DND-CAT Sector 5-ID Beamline Optimization"
	pack .title.text
	
	#-----------------------------------------------------------

	set statuswidth 30
	
	frame .sep1 -height 2 -borderwidth 1 -relief sunken
	pack .sep1 -expand yes -fill x -pady 5 -before .message_log
	
	frame .main
	pack .main -padx 10 -before .message_log
	
	set row 0
	
	label .main.mono_header -text Monochromator
	label .main.status_header -text Status
	label .main.old_header -text "Old Position"
	label .main.new_header -text "New Position"

	grid .main.mono_header         -row $row -column 0 -sticky w
	grid .main.status_header       -row $row -column 2
	grid .main.old_header          -row $row -column 3 -sticky w
	grid .main.new_header          -row $row -column 4 -sticky w
	
	incr row

	set tune [ $record_list get_record "tune" ]

	set thisfield .main.tune
	
	button ${thisfield}_button -text "Optimize Tune" \
			-background limegreen -command \
			[ code optimize_tune $record_list $tune $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $tune ]
	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row

	#-----------------------------------------------------------
	
	if { $MxOptimize::show_detune_button } {
		set detune_position [$record_list get_record "detune_position" ]
		set position [$detune_position receive_variable]

		set detune_ratio [$record_list get_record "detune_ratio" ]
		set percent [expr [$detune_ratio receive_variable]*100]

		set detune_side [$record_list get_record "detune_side" ]
		set side [expr ( ([$detune_side receive_variable]==1) ? \
				  ("right") : ("left") )]

		set thisfield .main.detune

		button ${thisfield}_button -text "Detune" \
			-background limegreen -command \
			[ code move_to_detune $record_list $tune $thisfield \
				.main.tune ] -state disabled
	
		label ${thisfield}_status -width $statuswidth \
			-text "Detune positon not determined"
	
		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
	
		incr row
	
		#-----------------------------------------------------------
	}
	
	label .main.blank2
	grid .main.blank2
	
	incr row
	
	label .main.slits -text Slits
	grid .main.slits -sticky w
	
	incr row
	
	set gs_vert [ $record_list get_record "bxv_center" ]

	set thisfield .main.gs_vert

	button ${thisfield}_button -text "Optimize Vertical Slit" \
	    -background limegreen -command \
	    [ code optimize_gs_vert $record_list $gs_vert $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text \
		[ optimize_get_position $gs_vert ]

	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	#-----------------------------------------------------------

	set gs_hor [ $record_list get_record "bxh_center" ]

	set thisfield .main.gs_hor
	
	button ${thisfield}_button -text "Optimize Horizontal Slit" \
			-background limegreen -command \
			[ code optimize_gs_hor $record_list $gs_hor $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $gs_hor ]
	label .main.gs_hor_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	#-----------------------------------------------------------
	
	frame .sep2 -height 2 -borderwidth 1 -relief sunken
	pack .sep2 -expand yes -fill x -pady 8 -before .message_log
	
	frame .bottom
	pack .bottom -before .message_log
	
	button .bottom.dismiss -text Dismiss -command exit
	pack .bottom.dismiss

	frame .sep3 -height 2 -borderwidth 1 -relief sunken
	pack .sep3 -expand yes -fill x -pady 8 -before .message_log
	
	wm deiconify .
}

proc disable_buttons {} {
	.main.tune_button configure -state disabled
	.main.gs_hor_button configure -state disabled
	.main.gs_vert_button configure -state disabled
	if { $MxOptimize::show_detune_button } {
		.main.detune_button configure -state disabled
	}
}

proc enable_buttons {} {
	.main.tune_button configure -state normal
	.main.gs_hor_button configure -state normal
	.main.gs_vert_button configure -state normal
}

proc optimize_tune { record_list motor guifield } {

	if { $MxOptimize::have_servo_control } {
		command_servo $record_list "servo_toggle" 0 
	}

	# store the amp settings in case the optimize scan uses autoscale_scaler
# This does not work because there is no amplifier records in libMxTcl
#	set amp_settings [read_amps $record_list]
	# reset all amps to gain = 1e4, offset = 0, time_constant = 0.0
	# this way I know the autoscale_scalars will work properly
# This does not work because there is no amplifier records in libMxTcl
#	reset_amps $record_list

	# The command "mp_amps reset" will reset all the amps and return a 
	# string that lists all the amps and their settings prior to the reset
	set status [ catch { eval exec mp_amps reset } amp_settings ]

	set result \
	 [perform_optimization $record_list $motor $guifield "fit_to_peak"]
#	 [eval exec optimize_peak_fitting 51 /home/maruser/oldscans/tune_scan.016]

	if { ($MxOptimize::show_detune_button) && ($result != "") } {
		scan $result "%d %g %d %g %g %g %g %g" \
		   fit_status euclidean_norm algorithm_info simple_position \
		   a1 a2 a3 a4

		if { [ catch { $record_list get_record "detune_ratio" } \
			detune_ratio ] } {
			set t "The variable 'detune_ratio' does not exist."
			append t "I cannot detune the mono without it."

			tk_dialog .detune_ratio_does_not_exist_dialog \
			"Detune Record Missing" \
			$t error 0 "OK"
			return
		}
		set ratio [$detune_ratio receive_variable]
		if { ($ratio > 0.9) || ($ratio < 0.3) } {
			set t "The detune ratio is out of range.  "
			append t "The allowed range is (0.3 - 0.9).  "
			append t "I will reset the detune_ratio to 0.8.  "

			tk_dialog .detune_out_of_range_dialog \
			"Detune Ratio Out of Range" \
			$t error 0 "OK"

			set ratio 0.8
			$detune_ratio send_variable $ratio
		}

		if { [ catch { $record_list get_record "detune_side" } \
			detune_side ] } {
			set t "The variable 'detune_side' does not exist."
			append t "I cannot detune the mono without it."

			tk_dialog .detune_side_does_not_exist_dialog \
			"Detune Record Missing" \
			$t error 0 "OK"
			return
		}
		set side [$detune_side receive_variable]
		if { ($side != -1) && ($side != 1) } {
			set t "The detune side is invalid.  "
			append t "Allowed values are -1 (left) or 1 (right).  "
			append t "I will reset the detune_side to -1.  "

			tk_dialog .detune_side_invalid_dialog \
			"Detune Side Invalid" \
			$t error 0 "OK"

			set side -1
			$detune_side send_variable $side
		}

		set y_at_a3 [expr $a1 + $a2]
		set offset \
		 [expr sqrt(-2 * pow($a4,2) * log((($y_at_a3*$ratio)-$a1)/$a2))]
		if { $side < 0 } {
			set position [expr abs($offset - $a3)]
		} else {
			set position [expr abs($offset + $a3)]
		}

		if { [ catch { $record_list get_record "detune_position" } \
			detune_position ] } {
			set t "The variable 'detune_position' does not exist."
			append t "I cannot detune the mono without it."

			tk_dialog .detune_position_does_not_exist_dialog \
			"Detune Record Missing" \
			$t error 0 "OK"
			return
		}
		set position [expr int($position)]

		if { ($position > 0) && ($position < 1000) } {
			$detune_position send_variable $position
			set percent [expr $ratio*100]
			set side [expr ( ($side==1) ? \ ("right") : ("left") )]
			.main.detune_button configure -state normal
			.main.detune_status configure \
				-text "Detune at $position ($percent% on $side)"
		}
	}

	# restore the amp settings to what they were before the scan
# This does not work because there is no amplifier records in libMxTcl
#	set_amps $record_list $amp_settings

	# The command "mp_amps set $amp_settings" will set all the amps 
	# to the settings as they were stored in amp_settings
	set status [ catch { eval exec mp_amps set \"$amp_settings\" } junk ]

	if { $MxOptimize::have_servo_control } {
		command_servo  $record_list "servo_toggle" 0 
	}

}

# This does not work because there is no amplifier records in libMxTcl
proc reset_amps {record_list} {
 set amp_list [ $record_list get_multiple_records "device" "amplifier" ".*" 0 ]
 foreach amp $amp_list {
#  puts "$amp"
  set amp_record [$record_list get_record $amp]
  set status [catch {$amp_record set_gain 1000.0}]
  if { $status != 0 } {
	set logfile_message "error in setting amplifier $amp gain to 1000"
        log_puts $logfile_message
        logfile_puts $logfile_message
  }
  set status [catch {$amp_record set_offset 0.0}]
  if { $status != 0 } {
	set logfile_message "error in setting amplifier $amp offset to 0"
        log_puts $logfile_message
        logfile_puts $logfile_message
  }
  set status [catch {$amp_record set_time_constant 0.0}]
  if { $status != 0 } {
	set logfile_message "error in setting amplifier $amp time_constant to 0"
        log_puts $logfile_message
        logfile_puts $logfile_message
  }
 }
}

# procedure that returns a list of all the current amplifier settings
# amp1_name amp1_gain amp1_offset amp2_name ...
# This does not work because there is no amplifier records in libMxTcl
proc read_amps {record_list} {
 set amp_list [ $record_list get_multiple_records "device" "amplifier" ".*" 0 ]
 set amp_settings ""
 foreach amp $amp_list {
#  puts "$amp"
  set amp_record [ $record_list get_record $amp ]
#  puts "$amp_record"
  lappend amp_settings $amp
  set gain [$amp_record get_gain]
  lappend amp_settings $gain
  set offset [$amp_record get_offset]
  lappend amp_settings $offset
  set time_constant [$amp_record get_time_constant]
  lappend amp_settings $time_constant
 }
# puts "$amp_settings"
 return $amp_settings
}

# procedure that sets the gain and offset for a list of amplifiers and settings
# argument is a single string "amp1_name amp1_gain amp1_offset amp2_name ..."
# This does not work because there is no amplifier records in libMxTcl
proc set_amps {record_list amp_settings} {
 set amp_list [ $record_list get_multiple_records "device" "amplifier" ".*" 0 ]
 foreach amp $amp_list {
#  puts "$amp"
  if {![catch {lsearch -exact $amp_settings $amp} amp_index]} { 
   set amp_record [$record_list get_record $amp]
   set gain [$amp_record get_gain]
   set offset [$amp_record get_offset]
   set time_constant [$amp_record get_time_constant]
   set old_gain [lindex $amp_settings [expr $amp_index + 1]]
   set old_offset [lindex $amp_settings [expr $amp_index + 2]]
   set old_time_constant [lindex $amp_settings [expr $amp_index + 3]]
   if {$gain != $old_gain} {
    set status [catch {$amp_record set_gain $old_gain}]
    if { $status != 0 } {
	set logfile_message "amplifier $amp gain was not properly reset from $gain to $old_gain"
        log_puts $logfile_message
        logfile_puts $logfile_message
    }
#    puts "$amp gain $gain $old_gain"
   }
   if {$offset != $old_offset} {
    set status [catch {$amp_record set_offset $old_offset}]
    if { $status != 0 } {
	set logfile_message "amplifier $amp offset was not properly reset from $offset to $old_offset"
        log_puts $logfile_message
        logfile_puts $logfile_message
    }
#    puts "$amp offset $offset $old_offset"
   }
   if {$time_constant != $old_time_constant} {
    set status [catch {$amp_record set_time_constant $old_time_constant}]
    if { $status != 0 } {
	set logfile_message "amplifier $amp time_constant was not properly reset from $time_constant to $old_time_constant"
        log_puts $logfile_message
        logfile_puts $logfile_message
    }
#    puts "$amp time_constant $time_constant $old_time_constant"
   }
  }
 }
}

proc optimize_gs_hor { record_list motor guifield } {

	# The command "mp_amps reset" will reset all the amps and return a 
	# string that lists all the amps and their settings prior to the reset
	set status [ catch { eval exec mp_amps reset } amp_settings ]

	perform_optimization $record_list $motor $guifield "fit_to_peak"

	# The command "mp_amps set $amp_settings" will set all the amps 
	# to the settings as they were stored in amp_settings
	set status [ catch { eval exec mp_amps set \"$amp_settings\" } junk ]

}


proc optimize_gs_vert { record_list motor guifield } {

	# The command "mp_amps reset" will reset all the amps and return a 
	# string that lists all the amps and their settings prior to the reset
	set status [ catch { eval exec mp_amps reset } amp_settings ]

	perform_optimization $record_list $motor $guifield "fit_to_peak"

	# The command "mp_amps set $amp_settings" will set all the amps 
	# to the settings as they were stored in amp_settings
	set status [ catch { eval exec mp_amps set \"$amp_settings\" } junk ]

}

proc move_to_detune { record_list tune guifield tuneguifield } {

	if { $MxOptimize::have_servo_control } {
		command_servo  $record_list "servo_toggle" 0 
	}

	set detune_position [$record_list get_record "detune_position" ]
	set position [$detune_position receive_variable]

	set detune_ratio [$record_list get_record "detune_ratio" ]
	set percent [expr [$detune_ratio receive_variable]*100]

	set detune_side [$record_list get_record "detune_side" ]
	set side [expr ( ([$detune_side receive_variable]==1) ? \
				  ("right") : ("left") )]

        ${tuneguifield}_status configure -fg yellow \
                                -text "Moving to new position"

        ${tuneguifield}_new_position configure -text $position

	set logfile_message "Moved tune to $position (detuned to $percent% on $side of tuning curve)"
        log_puts $logfile_message

        logfile_puts $logfile_message

        $tune move_absolute $position

        after 1000 \
	    check_final_detune_status \
		$guifield $tuneguifield $tune $percent $side "success"

	if { $MxOptimize::have_servo_control } {
		command_servo  $record_list "servo_toggle" 1 
	}

}

proc check_final_detune_status { \
	guifield tuneguifield motor percent side condition } {

	# See if the motor is still moving.

	set status [ catch { $motor is_busy } result ]

	if { $status == 0 } {

		if { $result != 0 } {

			after 1000 \
	  check_final_detune_status $guifield $motor $tuneguifield $condition

			return
		}
	} else {
		set condition "device_io_error"

		catch { $motor soft_abort }
	}

	if { $condition == "device_io_error" } {

		${tuneguifield}_status configure -fg red \
				-text "Motor device I/O error"

	} else {

		${tuneguifield}_status configure -fg limegreen -text \
			"Moved to fitted detune positon"

	}

        ${guifield}_status configure -fg limegreen \
				-text "$percent% of peak height on $side"

	enable_buttons
	.main.detune_button configure -state disabled
}

proc command_servo { record_list servo_record_name value } {
	
	set record [ $record_list get_record $servo_record_name ] 
	$record send_variable $value

}
