#
#  Copyright 1999-2005 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_optimize_window { record_list window } {

	global env

	if { $MxOptimize::beamline_name == "imca17bm" } {
		source "$env(MXDIR)/etc/imca17bm_feedback_interlock.tcl"
	}

	set toplevel_window $window.optimize_frame

	frame $toplevel_window

	if { $window == "" } {
		pack $toplevel_window -before .message_log
	} else {
		pack $toplevel_window
	}

	if { $window == "" } {
		set title $toplevel_window.title

		frame $title
		pack $title
	
		label $title.text -text "Sector 17 Beamline Optimization"
		pack $title.text
	
		frame $toplevel_window.sep1 \
				-height 2 -borderwidth 1 -relief sunken
		pack $toplevel_window.sep1 -expand yes -fill x -pady 5
	}
	
	set main $toplevel_window.main

	set MxOptimize::main $main

	#-----------------------------------------------------------

	set statuswidth 30
	
	frame $main
	pack $main -padx 10
	
	set row 0
	
	label $main.mono_header -text Monochromator
	label $main.status_header -text Status
	label $main.old_header -text "Old Position"
	label $main.new_header -text "New Position"

	grid $main.mono_header         -row $row -column 0 -sticky w
	grid $main.status_header       -row $row -column 2
	grid $main.old_header          -row $row -column 3 -sticky w
	grid $main.new_header          -row $row -column 4 -sticky w
	
	incr row

	set omega [ $record_list get_record "omega" ]

	set thisfield $main.omega
	
	button ${thisfield}_button -text "Optimize Omega" \
			-background limegreen -command \
		[ itcl::code optimize_omega $record_list $omega $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $omega ]
	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row

	set chi [ $record_list get_record "chi" ]

	set thisfield $main.chi
	
	button ${thisfield}_button -text "Optimize Chi" \
			-background limegreen -command \
			[ itcl::code optimize_chi $record_list $chi $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text [ optimize_get_position $chi ]
	label $main.chi_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row

	# Exit early for 17-BM if this is not the staff version.

	if { $MxOptimize::beamline_name == "imca17bm" } {
		if { $MxOptimize::program_name != "optimize_staff" } {
			finish_optimize_gui $record_list $toplevel_window
			return
		}
	}
	
	#-----------------------------------------------------------
	
	if { $MxOptimize::beamline_name == "imca17bm" } {

		# Code only for APS Sector 17-BM

		label $main.blankx
		grid $main.blankx
	
		incr row
	
		set focus [ $record_list get_record "focus" ]

		set thisfield $main.focus

		button ${thisfield}_button -text "Optimize Focus" \
			-background limegreen -command \
	[ itcl::code optimize_focus $record_list $focus $thisfield ]

		label ${thisfield}_status -width $statuswidth -text \
			"Not Yet Optimized"
	
		label ${thisfield}_old_position -text \
			[ optimize_get_position $focus ]

		label ${thisfield}_new_position -text "---"

		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
		grid ${thisfield}_old_position -row $row -column 3 -sticky e
		grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
		incr row

		# End of code only for APS Sector 17-BM
	}

	#-----------------------------------------------------------
	
	label $main.blank1
	grid $main.blank1
	
	incr row
	
	label $main.mirror -text Mirror
	grid $main.mirror -sticky w
	
	incr row

	if { $MxOptimize::beamline_name == "imca17id" } {
		set vert_mirror [ $record_list get_record "vmirror_z" ]
	} else {
		set vert_mirror [ $record_list get_record "mjack_vert" ]
	}

	set thisfield $main.vert_mirror
	
	button ${thisfield}_button -text "Optimize Vertical" \
	    -background limegreen -command \
    [ itcl::code optimize_vert_mirror $record_list $vert_mirror $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text \
		[ optimize_get_position $vert_mirror ]

	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e

	if { $MxOptimize::beamline_name == "imca17id" } {
		${thisfield}_button configure -state disabled
	}
	
	incr row
	
	#-----------------------------------------------------------
	
	label $main.blank2
	grid $main.blank2
	
	incr row
	
	label $main.slits -text Slits
	grid $main.slits -sticky w
	
	incr row

	set vert_slit [ $record_list get_record "bv_center" ]

	set thisfield $main.vert_slit

	button ${thisfield}_button -text "Optimize Vertical" \
	    -background limegreen -command \
	    [ itcl::code optimize_vert_slit $record_list $vert_slit $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text \
		[ optimize_get_position $vert_slit ]

	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	set horiz_slit [ $record_list get_record "bh_center" ]

	set thisfield $main.horiz_slit
	
	button ${thisfield}_button -text "Optimize Horizontal" \
	    -background limegreen -command \
    [ itcl::code optimize_horiz_slit $record_list $horiz_slit $thisfield ]
	
	label ${thisfield}_status -width $statuswidth -text "Not Yet Optimized"
	
	label ${thisfield}_old_position -text \
		[ optimize_get_position $horiz_slit ]

	label ${thisfield}_new_position -text "---"
	
	grid ${thisfield}_button       -row $row -column 0 -sticky w
	grid ${thisfield}_status       -row $row -column 2 -sticky w
	grid ${thisfield}_old_position -row $row -column 3 -sticky e
	grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
	incr row
	
	#-----------------------------------------------------------
	
	if { $MxOptimize::beamline_name == "imca17id" } {

		# Code only for APS Sector 17-ID

		label $main.blank3
		grid $main.blank3
	
		incr row
	
		label $main.table -text Table
		grid $main.table          -row $row -column 0 -sticky w

		label $main.halfslit_label -text "Half Slits"
		grid $main.halfslit_label -row $row -column 1
	
		incr row

		set table_vert [ $record_list get_record "tz" ]

		set thisfield $main.table_vert

		button ${thisfield}_button -text "Optimize Vertical" \
			-background limegreen -command \
	[ itcl::code optimize_table_vert $record_list $table_vert $thisfield ]

		label ${thisfield}_status -width $statuswidth -text \
			"Not Yet Optimized"
	
		label ${thisfield}_old_position -text \
			[ optimize_get_position $table_vert ]

		label ${thisfield}_new_position -text "---"

		set shutter_frame \
		    [ create_shutter_frame $thisfield $record_list "us_vert" ]
	
		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid $shutter_frame            -row $row -column 1 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
		grid ${thisfield}_old_position -row $row -column 3 -sticky e
		grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
		incr row

		set table_horiz [ $record_list get_record "ty" ]

		set thisfield $main.table_horiz

		button ${thisfield}_button -text "Optimize Horizontal" \
			-background limegreen -command \
	[ itcl::code optimize_table_horiz $record_list $table_horiz $thisfield ]

		label ${thisfield}_status -width $statuswidth -text \
			"Not Yet Optimized"
	
		label ${thisfield}_old_position -text \
			[ optimize_get_position $table_horiz ]

		label ${thisfield}_new_position -text "---"
	
		set shutter_frame \
		    [ create_shutter_frame $thisfield $record_list "us_horiz" ]
	
		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid $shutter_frame            -row $row -column 1 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
		grid ${thisfield}_old_position -row $row -column 3 -sticky e
		grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
		incr row

		set table_pitch [ $record_list get_record "tpitch" ]

		set thisfield $main.table_pitch

		button ${thisfield}_button -text "Optimize Pitch" \
			-background limegreen -command \
    [ itcl::code optimize_table_pitch $record_list $table_pitch $thisfield ]

		label ${thisfield}_status -width $statuswidth -text \
			"Not Yet Optimized"
	
		label ${thisfield}_old_position -text \
			[ optimize_get_position $table_pitch ]

		label ${thisfield}_new_position -text "---"
	
		set shutter_frame \
		    [ create_shutter_frame $thisfield $record_list "ds_vert" ]
	
		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid $shutter_frame            -row $row -column 1 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
		grid ${thisfield}_old_position -row $row -column 3 -sticky e
		grid ${thisfield}_new_position -row $row -column 4 -sticky e
	
		incr row

		set table_yaw [ $record_list get_record "tyaw" ]

		set thisfield $main.table_yaw

		button ${thisfield}_button -text "Optimize Yaw" \
			-background limegreen -command \
	[ itcl::code optimize_table_yaw $record_list $table_yaw $thisfield ]

		label ${thisfield}_status -width $statuswidth -text \
			"Not Yet Optimized"
	
		label ${thisfield}_old_position -text \
			[ optimize_get_position $table_yaw ]

		label ${thisfield}_new_position -text "---"
	
		set shutter_frame \
		    [ create_shutter_frame $thisfield $record_list "ds_horiz" ]
	
		grid ${thisfield}_button       -row $row -column 0 -sticky w
		grid $shutter_frame            -row $row -column 1 -sticky w
		grid ${thisfield}_status       -row $row -column 2 -sticky w
		grid ${thisfield}_old_position -row $row -column 3 -sticky e
		grid ${thisfield}_new_position -row $row -column 4 -sticky e

		incr row
	
		label $main.spacer -text ""
		grid $main.spacer          -row $row -column 0 -sticky w

		incr row

		button $main.table_control_start -text "Adjust Table Motors" \
				-background limegreen -command \
				[ itcl::code eval exec "adsc_table &" ]

		grid $main.table_control_start -row $row -column 0
	
		# End of code only for APS Sector 17-ID
	}

	#-----------------------------------------------------------

	finish_optimize_gui $record_list $toplevel_window
}
	
proc finish_optimize_gui { record_list toplevel_window } {

	#
	# For 17BM, we get the feedback records here.
	#

	if { $MxOptimize::beamline_name == "imca17bm" } {
		set Imca17bmFeedback::x_fb_enable \
			[ $record_list get_record "x_fb_enable" ]

		set Imca17bmFeedback::y_fb_enable \
			[ $record_list get_record "y_fb_enable" ]

		set Imca17bmFeedback::horiz_motor \
			[ $record_list get_record "chi" ]

		set Imca17bmFeedback::vert_motor \
			[ $record_list get_record "mjack_pitch" ]
	}

	#
	# If we are being run from Imcagui, we do not display a Dismiss
	# button here.
	#

	if { $toplevel_window != ".optimize_frame" } {
		return
	}

	frame $toplevel_window.sep2 -height 2 -borderwidth 1 -relief sunken
	pack $toplevel_window.sep2 \
		-expand yes -fill x -pady 8
	
	frame $toplevel_window.bottom
	pack $toplevel_window.bottom
	
	button $toplevel_window.bottom.dismiss -text Dismiss -command exit
	pack $toplevel_window.bottom.dismiss

	frame $toplevel_window.sep3 -height 2 -borderwidth 1 -relief sunken
	pack $toplevel_window.sep3 \
		-expand yes -fill x -pady 8
	
	if { $toplevel_window == "" } {
		wm deiconify .
	}
}

proc disable_button { button_name } {
	catch { $button_name configure -state disabled }
}

proc enable_button { button_name } {
	catch { $button_name configure -state normal }
}

proc disable_buttons {} {
	set main $MxOptimize::main

	disable_button $main.omega_button
	disable_button $main.chi_button
	disable_button $main.vert_mirror_button
	disable_button $main.vert_slit_button
	disable_button $main.horiz_slit_button

	if { $MxOptimize::beamline_name == "imca17id" } {
		disable_button $main.table_vert_button
		disable_button $main.table_horiz_button
		disable_button $main.table_pitch_button
		disable_button $main.table_yaw_button

		disable_button $main.table_control_start

		disable_button $main.table_vert_shutter_frame.removed
		disable_button $main.table_vert_shutter_frame.inserted

		disable_button $main.table_horiz_shutter_frame.removed
		disable_button $main.table_horiz_shutter_frame.inserted

		disable_button $main.table_pitch_shutter_frame.removed
		disable_button $main.table_pitch_shutter_frame.inserted

		disable_button $main.table_yaw_shutter_frame.removed
		disable_button $main.table_yaw_shutter_frame.inserted
	}
	if { $MxOptimize::beamline_name == "imca17bm" } {
		disable_button $main.focus_button
	}
}

proc enable_buttons {} {
	set main $MxOptimize::main

	enable_button $main.omega_button
	enable_button $main.chi_button

	if { $MxOptimize::beamline_name == "imca17bm" } {
		enable_button $main.focus_button
		enable_button $main.vert_mirror_button
	}

	enable_button $main.vert_slit_button
	enable_button $main.horiz_slit_button

	if { $MxOptimize::beamline_name == "imca17id" } {
		enable_button $main.table_vert_button
		enable_button $main.table_horiz_button
		enable_button $main.table_pitch_button
		enable_button $main.table_yaw_button

		enable_button $main.table_control_start

		enable_button $main.table_vert_shutter_frame.removed
		enable_button $main.table_vert_shutter_frame.inserted

		enable_button $main.table_horiz_shutter_frame.removed
		enable_button $main.table_horiz_shutter_frame.inserted

		enable_button $main.table_pitch_shutter_frame.removed
		enable_button $main.table_pitch_shutter_frame.inserted

		enable_button $main.table_yaw_shutter_frame.removed
		enable_button $main.table_yaw_shutter_frame.inserted
	}
}

proc create_shutter_frame { thisfield record_list shutter_name } {

	set shutter_record [ $record_list get_record $shutter_name ]

	set status ${shutter_name}_status

	set frame ${thisfield}_shutter_frame

	frame $frame

	label $frame.pad -text "   "
	pack $frame.pad -side left

	radiobutton $frame.removed -text "Removed" -variable $status -value 0

	pack $frame.removed -side left

	radiobutton $frame.inserted -text "Inserted" -variable $status -value 1

	pack $frame.inserted -side left

	set relay_status [ $shutter_record get_relay_status ]

	if { $relay_status == 0 } {
		$frame.removed select
	} else {
		$frame.inserted select
	}

	$frame.removed configure -command \
			[ itcl::code $shutter_record relay_command 0 ]

	$frame.inserted configure -command \
			[ itcl::code $shutter_record relay_command 1 ]

	return $frame
}

proc optimize_omega { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_chi { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_focus { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_horiz_slit { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_vert_slit { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_vert_mirror { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_peak"
}

proc optimize_table_vert { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_step"
}

proc optimize_table_horiz { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_step"
}

proc optimize_table_pitch { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_step"
}

proc optimize_table_yaw { record_list motor guifield } {

	perform_optimization $record_list $motor $guifield "fit_to_step"
}

